####################
# KERNEL

# define KERNEL_SRC
AC_DEFUN(
	[AC_WITH_KERNEL_SRC],
	[
		AC_ARG_WITH(
			[linux],
			AC_HELP_STRING(
				[--with-linux=<dir>],
				[ (default is /usr/src/linux)]),
			[KERNEL_SRC="$withval"],
			[KERNEL_SRC="/lib/modules/`uname -r`/build"])
		AC_SUBST([KERNEL_SRC])
	])

# check UTS_RELEASE in ${KERNEL_SRC}/include/linux/version.h
AC_DEFUN(
	[AC_CHECK_KERNEL_SRC],
	[
		AC_REQUIRE([AC_WITH_KERNEL_SRC]) dnl
		AC_MSG_CHECKING([linux kernel source in $KERNEL_SRC])
		KERNEL_RELEASE=`grep UTS_RELEASE ${KERNEL_SRC}/include/linux/version.h 2> /dev/null | cut -d\" -f2 || true`
		if test "$KERNEL_RELEASE" == ""; then
		        AC_MSG_RESULT([no])
			AC_MSG_ERROR([No include/linux/version.h in $KERNEL_SRC])
		fi
		AC_MSG_RESULT([yes])
		AC_SUBST([KERNEL_RELEASE])
	])

# define KERNEL_PATCHLEVEL
AC_DEFUN(
	[AC_KERNEL_PATCHLEVEL],
	[
		AC_REQUIRE([AC_CHECK_KERNEL_SRC]) dnl
		KERNEL_PATCHLEVEL=`echo $KERNEL_RELEASE | cut -d. -f1``echo $KERNEL_RELEASE | cut -d. -f2`
		AC_SUBST([KERNEL_PATCHLEVEL])
	])

# define KERNEL_SUBLEVEL
AC_DEFUN(
	[AC_KERNEL_SUBLEVEL],
	[
		AC_REQUIRE([AC_CHECK_KERNEL_SRC]) dnl
		[KERNEL_SUBLEVEL=`echo $KERNEL_RELEASE | cut -d. -f3 | sed 's/\([0-9]\+\).*/\1/'`]
		AC_SUBST([KERNEL_SUBLEVEL])
	])

# define KBUILD_TYPE
AC_DEFUN(
	[AC_KBUILD_TYPE],
	[
		AC_REQUIRE([AC_KERNEL_PATCHLEVEL]) dnl
		AC_REQUIRE([AC_KERNEL_SUBLEVEL]) dnl
		if test "$KERNEL_PATCHLEVEL" == 24; then
			KBUILD_TYPE="24"
		elif test "$KERNEL_PATCHLEVEL" == 26; then
			if test $KERNEL_SUBLEVEL -le 5; then
				KBUILD_TYPE="26BEFORE5"
			else
				KBUILD_TYPE="26AFTER6"
			fi
		fi
		AC_SUBST([KBUILD_TYPE])
	])

# define KERNEL_EXTRAVERSION
AC_DEFUN(
	[AC_KERNEL_EXTRAVERSION],
	[
		AC_REQUIRE([AC_CHECK_KERNEL_SRC]) dnl
		KERNEL_EXTRAVERSION=`echo $KERNEL_RELEASE | cut -d- -f2`
	])

# check KERNEL is 2.4.x or 2.6.x
AC_DEFUN(
	[AC_CHECK_KERNEL_VERSION],
	[
		AC_REQUIRE([AC_PROG_CPP]) dnl
		AC_REQUIRE([AC_CHECK_KERNEL_SRC]) dnl
		AC_MSG_CHECKING([linux kernel 2.4/2.6])
		AC_TRY_CPP(
			[
#include <$KERNEL_SRC/include/linux/version.h>
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0) && LINUX_VERSION_CODE < KERNEL_VERSION(2,5,0)
#elif LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0) && LINUX_VERSION_CODE < KERNEL_VERSION(2,7,0)
#else
#error
#endif
			],
			[AC_MSG_RESULT([yes ($KERNEL_RELEASE)])],
			[
				AC_MSG_RESULT([no ($KERNEL_RELEASE)])
				AC_MSG_ERROR([Kernel version not supported])
			])
	])

# check KERNEL has been patched with VMASPY
AC_DEFUN(
	[AC_CHECK_KERNEL_VMASPY],
	[
		AC_REQUIRE([AC_KERNEL_EXTRAVERSION]) dnl
		AC_MSG_CHECKING([kernel patched with vmaspy])
		if test "$KERNEL_EXTRAVERSION" = "vmaspy"; then
			AC_MSG_RESULT([yes])
		else
			AC_MSG_RESULT([no])
			AC_MSG_ERROR([vmaspy patched kernel required])
		fi
	])

####################
# GM

# define GM_SRC
AC_DEFUN(
	[AC_WITH_GM_SRC],
	[
		AC_ARG_WITH(
			[gm],
			AC_HELP_STRING(
				[--with-gm=<dir>],
				[ (default is /usr/src/gm)]),
			[GM_SRC="$withval"],
			[GM_SRC="/usr/src/gm"])
		AC_SUBST([GM_SRC])
	])

# check 
AC_DEFUN(
	[AC_CHECK_GM_SRC],
	[
		AC_REQUIRE([AC_WITH_GM_SRC]) dnl
		AC_MSG_CHECKING([GM source in $GM_SRC])
		GM_VERSION=`grep _gm_version ${GM_SRC}/libgm/_gm_version.c 2> /dev/null | cut -d\" -f2 || true`
		if test "$GM_VERSION" == ""; then
		        AC_MSG_RESULT([no])
			AC_MSG_ERROR([No libgm/_gm_version.c in in $GM_SRC])
		fi
		AC_MSG_RESULT([yes ($GM_VERSION)])
	])

# check api version
# $1 is version 1_2 or 2_0_6 for example
# $2 is the feature name
AC_DEFUN(
	[AC_CHECK_GM_API_VERSION],
	[
		AC_REQUIRE([AC_CHECK_GM_SRC]) dnl
		AC_MSG_CHECKING([for $2 support in GM API ($1 required)])
		AC_TRY_CPP(
			[
#define GM_API_VERSION GM_API_VERSION_$1
#include <$GM_SRC/include/gm.h>
			],
			[AC_MSG_RESULT([yes])],
			[
				AC_MSG_RESULT([no])
				AC_MSG_ERROR([GM version too old])
			])
	])

# check gm
AC_DEFUN(
	[AC_CHECK_GM],
	[
		AC_CHECK_GM_API_VERSION(
			[2_0_6],
			[gm_register_memory_ex])
	])

########################################
# REGISTRATION CACHE

# define enable_regcache
AC_DEFUN(
	[AC_REGCACHE_ENABLE],
	[AC_ARG_ENABLE(
		[regcache],
		AC_HELP_STRING(
			[--disable-regcache],
			[(default is enabled)]),
		[
			enable_regcache="$enableval"
			AC_MSG_RESULT([activating registration cache... $enableval])
		],
		[enable_regcache=yes])
	])

# define REGCACHE
AC_DEFUN(
	[AC_REGCACHE],
	[
		AC_REQUIRE([AC_REGCACHE_ENABLE]) dnl
		if test "$enable_regcache" != no; then
			AC_CHECK_KERNEL_VMASPY
			AC_DEFINE(
				[CONFIG_GMKRC_REGCACHE],
				1,
				[Define to activate registration cache])
		fi
		AC_SUBST([CONFIG_GMKRC_REGCACHE], [$enable_regcache])
	])

########################################
# AMOUNT

# define enable_amount
AC_DEFUN(
	[AC_AMOUNT_ENABLE],
	[AC_ARG_ENABLE(
		[amount],
	        AC_HELP_STRING(
			[--enable-amount],
			[(default is no)]),
        	[
			enable_amount="$enableval"
			AC_MSG_RESULT([fixing amount of registrable pages... $enableval])
		],
	        [enable_amount=no])
	])

# define AMOUNT
AC_DEFUN(
	[AC_AMOUNT],
	[
		AC_REQUIRE([AC_AMOUNT_ENABLE]) dnl
		if test "$enable_amount" != no; then
			AC_DEFINE(
				[CONFIG_GMKRC_AMOUNT],
				[1],
				[Define to limit the amount of registrable pages])
			if test "$enable_amount" = yes; then
				amount_level=10;
			else
				amount_level=$enable_amount
			fi
			AC_DEFINE_UNQUOTED(
				[CONFIG_GMKRC_AMOUNT_LEVEL],
				[$amount_level],
				[Define to the amount of registrable pages])
		fi
	])

########################################
# DEBUG

# define enable_debug
AC_DEFUN(
	[AC_DEBUG_ENABLE],
	[AC_ARG_ENABLE(
		[debug],
	        AC_HELP_STRING(
			[--enable-debug],
			[(default is no)]),
        	[
			enable_debug="$enableval"
			AC_MSG_RESULT([activating debug... $enableval])
		],
	        [enable_debug=no])
	])

# define DEBUG
AC_DEFUN(
	[AC_DEBUG],
	[
		AC_REQUIRE([AC_DEBUG_ENABLE]) dnl
		if test "$enable_debug" != no; then
			AC_DEFINE(
				[CONFIG_GMKRC_DEBUG],
				[1],
				[Define to activate debug printing])
			if test "$enable_debug" = yes; then
				debug_level=0
			else
				debug_level=$enable_debug
			fi
			AC_DEFINE_UNQUOTED(
				[CONFIG_GMKRC_DEBUG_LEVEL],
				[$debug_level],
				[Define to the initial value of debug printing level])
		fi
	])
