#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/sysctl.h>

#include "gmkrc.h"
#include "gmkrc_vmaspy.h"
#include "gmkrc_regcache.h"
#include "gmkrc_internal.h"
#include "gmkrc_messages.h"


struct gmkrc_port gmkrc_ports[GM_NUM_PORTS];


#if CONFIG_GMKRC_DEBUG

/* debug mask */
int gmkrc_debug = CONFIG_GMKRC_DEBUG_LEVEL;

#if GMKRC_LINUX_24
MODULE_PARM(gmkrc_debug, "i");
#elif GMKRC_LINUX_26
#include <linux/moduleparam.h>
module_param(gmkrc_debug, int, 0644);
#endif


#define CTL_DEBUG_GMKRC 545 /* random number */

static struct ctl_table_header *gmkrc_ctl_header;

static struct ctl_table gmkrc_debug_ctl_table[] = {
  {CTL_DEBUG_GMKRC, "gmkrc", &gmkrc_debug, sizeof(gmkrc_debug),
   0644,
   NULL, &proc_dointvec},
  {0}
};

static struct ctl_table gmkrc_root_ctl_table[] = {
  {CTL_DEBUG, "debug", NULL, 0, 0555, gmkrc_debug_ctl_table},
  {0},
};

#endif /* CONFIG_GMKRC_DEBUG */


static int
gmkrc_init(void)
{
  int error = 0;
  int i;

  for(i=0; i<GM_NUM_PORTS; i++) {
    gmkrc_ports[i].port = NULL;
    sema_init(&gmkrc_ports[i].sem, 1);
  }

#if CONFIG_GMKRC_REGCACHE
  error = gmkrc_regcache_init();
  if ( error < 0 )
    goto error;
#endif /* CONFIG_GMKRC_REGCACHE */

#if CONFIG_GMKRC_DEBUG
  gmkrc_ctl_header = register_sysctl_table(gmkrc_root_ctl_table, 0);
  if ( !gmkrc_ctl_header ) {
    ERROR("Cannot register sysctl table\n");
    goto error_before_sysctl;
  }
#endif /* CONFIG_GMKRC_DEBUG */

  NOTICE("initialized\n");
  return error;

#if CONFIG_GMKRC_REGCACHE
#if CONFIG_GMKRC_DEBUG
 error_before_sysctl:
  gmkrc_regcache_exit();
#endif
 error:
  return error;
#else /* CONFIG_GMKRC_REGCACHE */
#if CONFIG_GMKRC_DEBUG
 error_before_sysctl:
  return error;
#endif
#endif /* ~CONFIG_GMKRC_REGCACHE */
}


static void
gmkrc_exit(void)
{
#if CONFIG_GMKRC_REGCACHE
  gmkrc_regcache_exit();
  gmkrc_vma_spies_exit();
#endif /* CONFIG_GMKRC_REGCACHE */

#if CONFIG_GMKRC_DEBUG
  unregister_sysctl_table(gmkrc_ctl_header);
#endif /* CONFIG_GMKRC_DEBUG */

  NOTICE("finished\n");
}


MODULE_AUTHOR("Brice Goglin <Brice.Goglin@ens-lyon.fr>");
MODULE_LICENSE("GPL");

module_init(gmkrc_init);
module_exit(gmkrc_exit);


EXPORT_SYMBOL(gmkrc_open);
EXPORT_SYMBOL(gmkrc_close);
EXPORT_SYMBOL(gmkrc_register_memory);
EXPORT_SYMBOL(gmkrc_deregister_memory);
EXPORT_SYMBOL(gmkrc_send_with_callback);
EXPORT_SYMBOL(gmkrc_provide_receive_buffer_with_tag);
